
/*
  Copyright (C) 2002, 2004  Dmitry V. Levin <ldv@altlinux.org>

  child handler

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <error.h>
#include <paths.h>
#include <sys/stat.h>
#include <linux/limits.h>

#include "conv.h"
#include "consolehelper.h"

/*
 * child process
 */
void
do_child(int in, int out, char *const argv[])
{
	int     fd, max = sysconf(_SC_OPEN_MAX);
	const char *prog = PRIVDIR "/auth";
	struct stat stb;

	/*
	 * close unneeded file descriptors
	 */
	if (max < NR_OPEN)
		max = NR_OPEN;

	for (fd = 1 + STDERR_FILENO; fd < max; ++fd)
		if ((fd != in) && (fd != out))
			close(fd);

	/*
	 * open/reassign descriptors
	 */
	if (fstat(STDIN_FILENO, &stb) < 0)
	{
		fd = open(_PATH_DEVNULL, O_RDONLY);
		if (fd < 0)
			error(EXIT_FAILURE, errno, "open: %s", _PATH_DEVNULL);
		if (fd != STDIN_FILENO)
		{
			if (dup2(fd, STDIN_FILENO) != STDIN_FILENO)
				error(EXIT_FAILURE, errno, "dup2");
			if (close(fd) < 0)
				error(EXIT_FAILURE, errno, "close");
		}
	}

	if (fstat(STDOUT_FILENO, &stb) < 0)
	{
		fd = open(_PATH_DEVNULL, O_WRONLY);
		if (fd < 0)
			error(EXIT_FAILURE, errno, "open: %s", _PATH_DEVNULL);
		if (fd != STDOUT_FILENO)
		{
			if (dup2(fd, STDOUT_FILENO) != STDOUT_FILENO)
				error(EXIT_FAILURE, errno, "dup2");
			if (close(fd) < 0)
				error(EXIT_FAILURE, errno, "close");
		}
	}

	if (fstat(STDERR_FILENO, &stb) < 0)
	{
		fd = open(_PATH_DEVNULL, O_WRONLY);
		if (fd < 0)
			error(EXIT_FAILURE, errno, "open: %s", _PATH_DEVNULL);
		if (fd != STDERR_FILENO)
		{
			if (dup2(fd, STDERR_FILENO) != STDERR_FILENO)
				error(EXIT_FAILURE, errno, "dup2");
			if (close(fd) < 0)
				error(EXIT_FAILURE, errno, "close");
		}
	}

	if (fstat(CHILDIN_FILENO, &stb) < 0)
	{
		if (in == CHILDIN_FILENO)
			error(EXIT_FAILURE, errno, "fstat: CHILDIN_FILENO");
		if (dup2(in, CHILDIN_FILENO) != CHILDIN_FILENO)
			error(EXIT_FAILURE, errno, "dup2");
		if (close(in) < 0)
			error(EXIT_FAILURE, errno, "close");
	}

	if (fstat(CHILDOUT_FILENO, &stb) < 0)
	{
		if (out == CHILDOUT_FILENO)
			error(EXIT_FAILURE, errno, "fstat: CHILDOUT_FILENO");
		if (dup2(out, CHILDOUT_FILENO) != CHILDOUT_FILENO)
			error(EXIT_FAILURE, errno, "dup2");
		if (close(out) < 0)
			error(EXIT_FAILURE, errno, "close");
	}

	execv(prog, argv);
	error(EXIT_SUCCESS, errno, "execv: %s", prog);
	if (dup2(CHILDOUT_FILENO, STDERR_FILENO) == STDERR_FILENO)
		fprintf(stderr, ":%s: %s", prog, strerror(errno));
	exit(EXIT_FAILURE);
}
