
/*
  Copyright (C) 2002-2005  Dmitry V. Levin <ldv@altlinux.org>

  conversation

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <error.h>
#include <sys/wait.h>
#include <libintl.h>

#include "conv.h"
#include "dialogs.h"

#define _(arg) gettext(arg)

static void
block_signal_handler(int no, int what)
{
	sigset_t unblock;

	sigemptyset(&unblock);
	sigaddset(&unblock, no);
	if (sigprocmask(what, &unblock, 0) < 0)
		error(EXIT_FAILURE, errno, "sigprocmask");
}

static volatile pid_t child_pid;	/* child process PID */
static int child_rc = EXIT_FAILURE;	/* child return code */

/*
 * handler to control return status of the child process
 */
static void
sigchld_handler(int signo)
{
	int     status;
	pid_t   old_child;

#ifdef DEBUG
	printf("SIGCHLD, child pid=%d\n", child_pid);
#endif
	if (!child_pid)
		return;

	if (waitpid(child_pid, &status, WNOHANG) != child_pid)
		return;

	signal(SIGCHLD, SIG_DFL);

	old_child = child_pid;
	child_pid = 0;

	if (WIFEXITED(status))
	{
#ifdef DEBUG
		if (WEXITSTATUS(status))
			printf("child %d exited with return code %d.\n",
			       old_child, WEXITSTATUS(status));
		else
			printf("child %d exited normally.\n", old_child);
#endif
		child_rc = WEXITSTATUS(status);
		return;
	}

	if (WIFSIGNALED(status))
	{
#ifdef DEBUG
		error(EXIT_SUCCESS, errno,
		      "child %d terminated with signal %d.\n", old_child,
		      WTERMSIG(status));
#endif
		child_rc = 128 + WTERMSIG(status);
		return;
	}
}

/*
 * main conversation function
 */
int
do_conv(char *const argv[])
{
	int     in[2], out[2], rc = 0;
	const char *answer;

	if ((pipe(in) < 0) || (pipe(out) < 0))
		error(EXIT_FAILURE, errno, "pipe");

	signal(SIGPIPE, SIG_IGN);
	block_signal_handler(SIGCHLD, SIG_BLOCK);

	child_pid = fork();
	if (child_pid < 0)
		error(EXIT_FAILURE, errno, "fork");

	if (!child_pid)
	{
		if ((close(in[1]) < 0) || (close(out[0]) < 0))
			error(EXIT_FAILURE, errno, "close");

		block_signal_handler(SIGCHLD, SIG_UNBLOCK);

		do_child(in[0], out[1], argv);
	}

	if ((close(in[0]) < 0) || (close(out[1]) < 0))
		error(EXIT_FAILURE, errno, "close");

	signal(SIGCHLD, sigchld_handler);
	block_signal_handler(SIGCHLD, SIG_UNBLOCK);

	answer = do_parent(in[1], out[0]);

	if ((close(in[1]) < 0) || (close(out[0]) < 0))
		error(EXIT_FAILURE, errno, "close");

	if (answer)
	{
		if (display_error(_(answer && *answer ? answer : "Conversation error."), 0))
			rc = -1;
		else
			rc = EXIT_FAILURE;
	}

	while (child_pid)
		usleep(100000);
	return rc ? : child_rc;
}
