
/*
  Copyright (C) 2002,2003  Dmitry V. Levin <ldv@altlinux.org>

  dialog functions

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <unistd.h>
#include <libintl.h>
#include <gtk/gtk.h>

#include "dialogs.h"
#include "gui.h"
#include "tui.h"

int     setresgid(gid_t rgid, gid_t egid, gid_t sgid);

static int
gui_available(void)
{
	static int tested, gui_avail;

	if (!tested)
	{
		const char *display = getenv("DISPLAY");
		int     argc = 1;
		char  **argv;
		gid_t   gid = getgid();

		setresgid(gid, gid, gid);	/* bypass gtk check_setugid */

		argv = calloc(argc + 1, sizeof(char *));
		argv[0] = "consolehelper";

		if (display && *display && gtk_init_check(&argc, &argv))
		{
			bind_textdomain_codeset(PACKAGE, "UTF-8");
			gtk_set_locale();
			gui_avail = 1;
		}
		tested = 1;
	}
	return gui_avail;
}

int
display_error(const char *message, const char *title)
{
	if (gui_available())
		return gui_display_error(message, title);
	tui_display_error(message, title);
	return 0;
}

const char *
get_user_pw(const char *username)
{
	if (gui_available())
		return gui_get_user_pw(username);
	return tui_get_user_pw(username);
}
