
/*
  Copyright (C) 2002, 2004  Dmitry V. Levin <ldv@altlinux.org>

  parent handler

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <error.h>
#include <utmp.h>
#include <pwd.h>

#include "conv.h"
#include "dialogs.h"
#include "consolehelper.h"

IO_LOOP(write_loop, write, const)
IO_LOOP(read_loop, read,)

/*
 * parent process
 */
const char *do_parent(int auth_in, int auth_out)
{
	ssize_t count;

	/*
	 * read username
	 */
	{
		char    name[BUFSIZ];

		count = read(auth_out, name, sizeof(name) - 1);
		if (count <= 0)
			return 0;

		name[count] = '\0';

		if (name[0] == ':')
			return strdup(name + 1) ? : strerror(ENOMEM);
		if (!getpwnam(name))
			return "username lookup failure";
		else
		{
			char   *pw = (char *) get_user_pw(name);
			size_t  len = strlen(pw);

			count = write_loop(auth_in, pw, len);
			memset(pw, 0, len);
			free(pw);
			if (count != len)
				return "error writing password to peer";
		}
	}

	/*
	 * wait for result
	 */
	{
		char    buf[BUFSIZ];

		count = read_loop(auth_out, buf, sizeof(buf) - 1);
		if (count <= 0)
			return 0;

		buf[count] = '\0';
		return strdup(buf[0] ==
			      ':' ? buf + 1 : buf) ? : strerror(ENOMEM);
	}
}
