
/*
  Copyright (C) 2002, 2005  Dmitry V. Levin <ldv@altlinux.org>

  TUI implementation of dialog functions

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <locale.h>
#include <libintl.h>

#include "tui.h"

#define _(arg) gettext(arg)

void
tui_display_error(const char *message, const char *title)
{
	printf("%s: %s\n", title ? : "consolehelper", message);
}

/*
 * ask for password
 */
static const char *
get_pw(const char *title, const char *message, const char *prompt)
{
	char   *pw, *res;

	puts(message);
	pw = getpass(prompt);

	if (!pw)
		return pw;
	if (asprintf(&res, "%s\n", pw) < 0)
		error(EXIT_FAILURE, errno, "asprintf");

	return res;
}

/*
 * ask for user password
 */
const char *
tui_get_user_pw(const char *username)
{
	char   *message1, *message2;
	const char *pw;

	if (asprintf
	    (&message1,
	     _
	     ("In order to run \"%s\",\nadditional information is required."),
	     program_invocation_short_name) < 0)
		error(EXIT_FAILURE, errno, "asprintf");

	if (asprintf(&message2, _("Enter password for %s %s: "),
		     (strcmp(username, "root") ? _("user") :
		      _("administrator")), username) < 0)
		error(EXIT_FAILURE, errno, "asprintf");

	pw = get_pw(_("Enter password"), message1, message2);

	free(message2);
	free(message1);

	if (!pw)
		exit(EXIT_FAILURE);

	return pw;
}
