
/*
  Copyright (C) 2002, 2004  Dmitry V. Levin <ldv@altlinux.org>

  pam conversation

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "serv_pam.h"
#include <security/pam_misc.h>

#include "server.h"

static char *
read_pass(const char *user)
{
	ssize_t len = strlen(user);

	if (write_client(user, len, 0) != len)
		return 0;

	return read_client();
}

int
do_pam_conversation(int num_msg, const struct pam_message **msg,
		    struct pam_response **resp, void *appdata_ptr)
{
	struct pam_response *reply;
	int     count;
	const char *user = (const char *) appdata_ptr;

	if (!user || !*user)
		return PAM_CONV_ERR;

	/* PAM will free this later */
	reply = calloc(num_msg, sizeof(*reply));
	if (!reply)
		return PAM_CONV_ERR;

	for (count = 0; count < num_msg; ++count)
	{
		switch (msg[count]->msg_style)
		{
			case PAM_PROMPT_ECHO_OFF:
				reply[count].resp = read_pass(user);
				reply[count].resp_retcode =
					reply[count].resp ? PAM_SUCCESS :
					PAM_CONV_ERR;
				break;
			case PAM_PROMPT_ECHO_ON:
			case PAM_ERROR_MSG:
			case PAM_TEXT_INFO:
				reply[count].resp = strdup("");
				reply[count].resp_retcode = PAM_SUCCESS;
				break;
			default:
				free(reply);
				return PAM_CONV_ERR;
		}
	}

	*resp = reply;

	return PAM_SUCCESS;
}
