
/*
 */

#include <stdlib.h>

#include "serv_pam.h"
#include <security/pam_misc.h>

#include "server.h"

/*
 * construct environment for PAM (and hence the shell)
 */

extern char **environ;

/*
 * We will attempt to transcribe the following env variables
 * independent of whether we keep the whole environment. Others will
 * be set elsewhere: either in modules; or after the identity of the
 * user is known.
 */

const char *posix_env[] = {
	"LANG",
	"LC_CTYPE",
	"LC_NUMERIC",
	"LC_TIME",
	"LC_COLLATE",
	"LC_MONETARY",
	"LC_MESSAGES",
	"LC_PAPER",
	"LC_NAME",
	"LC_ADDRESS",
	"LC_TELEPHONE",
	"LC_MEASUREMENT",
	"LC_IDENTIFICATION",
	"LC_ALL",
	"TZ",
	"DISPLAY",

	/* global proxy settings */
	"http_proxy",
	"HTTP_PROXY",
	"https_proxy",
	"HTTPS_PROXY",
	"ftp_proxy",
	"FTP_PROXY",

	NULL
};

int
make_env(pam_handle_t * pamh, int keep_env)
{
	int     rc;

	if (keep_env)
	{
		/* preserve the original environment */
		rc = pam_misc_paste_env(pamh, (const char *const *) environ);

	} else
	{
		int     i;

		/* we always transcribe some variables anyway */
		{
			const char *tmpe = getenv("TERM") ? : "dumb";

			rc = pam_misc_setenv(pamh, "TERM", tmpe, 0);
			D(("pam_misc_setenv: TERM=%s: rc=%d", tmpe, rc));
			if (rc != PAM_SUCCESS)
				return rc;
		}

		/* also propogate the POSIX specific ones */
		for (i = 0; rc == PAM_SUCCESS && posix_env[i]; ++i)
		{
			const char *tmpe = getenv(posix_env[i]);

			if (tmpe != NULL)
			{
				rc = pam_misc_setenv(pamh, posix_env[i],
						     tmpe, 0);
				D(("pam_misc_setenv: %s=%s: rc=%d",
				   posix_env[i], tmpe, rc));
				if (rc != PAM_SUCCESS)
					return rc;
			}
		}
	}

	return rc;		/* how did we do? */
}
