
/*
  Copyright (C) 2002  Dmitry V. Levin <ldv@altlinux.org>

  fds check

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <linux/limits.h>

#include "server.h"
#include "consolehelper.h"

static int
get_max_fds()
{
	static int max_fds;

	if (!max_fds)
	{
		max_fds = sysconf(_SC_OPEN_MAX);
		if (max_fds < NR_OPEN)
			max_fds = NR_OPEN;
	}
	return max_fds;
}

void
check_fds(void)
{
	int     fd, max = get_max_fds();
	struct stat stb;

	/*
	 * check for necessary descriptors.
	 */
	for (fd = STDERR_FILENO; fd <= CHILDOUT_FILENO; ++fd)
		if (fstat(fd, &stb) < 0)
			error(EXIT_FAILURE, errno, "fstat: %d", fd);

	/*
	 * check for unneeded descriptors.
	 */
	for (; fd < max; ++fd)
		if (close(fd) == 0)
			error(EXIT_FAILURE, errno, "unneeded fd: %d", fd);
}

void
cleanup_fds(void)
{
	int     fd, max = get_max_fds();

	/*
	 * cleanup unneeded descriptors.
	 */
	for (fd = STDERR_FILENO + 1; fd < max; ++fd)
		close(fd);
}
