
/*
  Copyright (C) 2002  Dmitry V. Levin <ldv@altlinux.org>

  set user credentials

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>

#include "server.h"
#include "serv_pam.h"
#include <security/pam_misc.h>

int
set_user_credentials(pam_handle_t * pamh, const char *user, uid_t * uid)
{
	int     rc;
	const char *path;
	const struct passwd *pw = getpwnam(user);

	if (!pw)
		return PAM_USER_UNKNOWN;

	*uid = pw->pw_uid;

	if (initgroups(pw->pw_name, pw->pw_gid) < 0 || setgid(pw->pw_gid) < 0)
		return PAM_PERM_DENIED;

	/* set PATH, LOGNAME, HOME according to user uid */
	path = pw->pw_uid ?
		"/bin:/usr/bin:/usr/X11R6/bin:/usr/local/bin" :
		"/sbin:/usr/sbin:/usr/local/sbin:/bin:/usr/bin:/usr/X11R6/bin:/usr/local/bin";

	rc = pam_misc_setenv(pamh, "PATH", path, 0);
	if (rc != PAM_SUCCESS)
		return PAM_CRED_ERR;

	rc = pam_misc_setenv(pamh, "LOGNAME", user, 0);
	if (rc != PAM_SUCCESS)
		return PAM_CRED_ERR;

	rc = pam_misc_setenv(pamh, "HOME", pw->pw_dir, 0);
	if (rc != PAM_SUCCESS)
		return PAM_CRED_ERR;

	endpwent();

	rc = pam_setcred(pamh, PAM_ESTABLISH_CRED);
	if (rc != PAM_SUCCESS)
		return rc;

	return PAM_SUCCESS;
}
