
/*
 * shvar.h
 *
 * Interface for non-destructively reading/writing files containing
 * only shell variable declarations and full-line comments.
 *
 * Includes explicit inheritance mechanism intended for use with
 * Red Hat Linux ifcfg-* files.  There is no protection against
 * inheritance loops; they will generally cause stack overflows.
 * Furthermore, they are only intended for one level of inheritance;
 * the value setting algorithm assumes this.
 *
 * Copyright 1999 Red Hat, Inc.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef CONSHELP_SHVAR_H
#define CONSHELP_SHVAR_H

#ifdef __cplusplus
extern  "C"
{
#endif				/* __cplusplus */

	typedef struct _shvarList shvarList;
	struct _shvarList
	{
		const char *key;
		const char *value;
		shvarList *next;
	};

	typedef struct _shvarFile shvarFile;
	struct _shvarFile
	{
		int     fd;     /* read-only */
		char   *arena;
		shvarList *lineList;	/* read-only */
	};

/* Open the file <name>, return shvarFile on success, NULL on failure */
	shvarFile *svNewFile(const char *name);

/* Get the value associated with the key.
 * The char* returned MUST be freed by the caller.
 */
	const char   *svGetValue(shvarFile * s, const char *key);

/* return 1 if <key> resolves to any truth value (e.g. "yes", "y", "true")
 * return 0 if <key> resolves to any non-truth value (e.g. "no", "n", "false")
 * return <def> otherwise
 */
	int     svTrueValue(shvarFile * s, const char *key, int def);

/* Close the file descriptor (if open) and delete the shvarFile.
 * Returns -1 on error and 0 on success.
 */
	int     svCloseFile(shvarFile * s);

#ifdef __cplusplus
}
#endif				/* __cplusplus */

#endif				/* ! CONSHELP_SHVAR_H */
