
/*
  Copyright (C) 2002, 2004  Dmitry V. Levin <ldv@altlinux.org>

  wait for child

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>

#include "server.h"

int
wait_for_child(pid_t child)
{
	int     status;

	signal(SIGCHLD, SIG_DFL);
	if (waitpid(child, &status, 0) != child)
		return EXIT_FAILURE;

	if (WIFEXITED(status))
	{
#ifdef DEBUG
		if (WEXITSTATUS(status))
			printf("child %d exited with return code %d.\n",
			       child, WEXITSTATUS(status));
		else
			printf("child %d exited normally.\n", child);
#endif
		return WEXITSTATUS(status);
	}

	if (WIFSIGNALED(status))
	{
#ifdef DEBUG
		error(EXIT_SUCCESS, errno,
		      "child %d terminated with signal %d.\n", child,
		      WTERMSIG(status));
#endif
		return 128 + WTERMSIG(status);
	}

	return EXIT_FAILURE;
}
